
// You can redistribute this file and/or modify it under the terms of the MIT License.
// 
// Copyright (c) 2015 Saurabh Garg
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
// 
// Author(s): Saurabh Garg
// Email: saurabhgarg@mysoc.net

#ifndef _SUTILS_TIMER_H_
#define _SUTILS_TIMER_H_

struct TimerPriv;

class Timer
{
public:
	
	enum TimeUnit
	{
		TimeInSeconds      = 1,
		TimeInMilliSeconds = 1000,
		TimeInMicroSeconds = 1000000
	};
	
public:
	
	Timer();
	~Timer();
	
	// On Windows, returns true if high performance timer is available.
	// On Linux, always returns true.
	bool IsTimerAvailable();
	
	// Start the timer.
	void Start();
	
	// Stop the timer and return the time elapsed since the timer was started.
	double Stop(TimeUnit timeUnit = TimeInMilliSeconds);
	
	// Get the time elapsed since Start() was called.
	double TimeElapsedSinceStart(TimeUnit timeUnit = TimeInMilliSeconds);
	
	// Get the total time elapsed between Start() and Stop().
	double TotalTimeElasped(TimeUnit timeUnit = TimeInMilliSeconds);
	
private:
	
	TimerPriv* mPD;
};

#endif // _SUTILS_TIMING_H_
