
// You can redistribute this file and/or modify it under the terms of the MIT License.
// 
// Copyright (c) 2015 Saurabh Garg
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
// 
// Author(s): Saurabh Garg
// Email: saurabhgarg@mysoc.net

#include "Timer.h"
#include <time.h>

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //
struct TimerPriv
{
	struct timespec mStartTime;
	struct timespec mEndTime;
};
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //


// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //
// Constructor.
Timer::Timer()
{
	mPD = new TimerPriv;
}
Timer::~Timer()
{
	delete mPD;
}
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //


// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //
bool Timer::IsTimerAvailable()
{
	return true;
}
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //


// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //
// Start the timer.
void Timer::Start()
{
	clock_gettime(CLOCK_MONOTONIC, &mPD->mStartTime);
}
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //


// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //
// Stop the timer.
double Timer::Stop(TimeUnit timeUnit)
{
	clock_gettime(CLOCK_MONOTONIC, &mPD->mEndTime);
	return TotalTimeElasped(timeUnit);
}
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //


// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //
double Timer::TimeElapsedSinceStart(TimeUnit timeUnit)
{
	struct timespec _currTime;
	clock_gettime(CLOCK_MONOTONIC, &_currTime);
	
	// tv_sec is time in seconds.
	// tv_nsec is time in nano seconds.
	return timeUnit * ((_currTime.tv_sec + _currTime.tv_nsec*1e-9) - (mPD->mStartTime.tv_sec + mPD->mStartTime.tv_nsec*1e-9));
}
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //


// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //
double Timer::TotalTimeElasped(TimeUnit timeUnit)
{
	// tv_sec is time in seconds.
	// tv_nsec is time in nano seconds.
	return timeUnit * ((mPD->mEndTime.tv_sec + mPD->mEndTime.tv_nsec*1e-9) - (mPD->mStartTime.tv_sec + mPD->mStartTime.tv_nsec*1e-9));
}
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //
